@@ -13,6 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+from typing import MutableMapping, MutableSequence
+
 import proto  # type: ignore
 
 from google.cloud.datastore_v1.types import aggregation_result
@@ -63,24 +65,24 @@ class LookupRequest(proto.Message):
             string '' to refer the default database.
         read_options (google.cloud.datastore_v1.types.ReadOptions):
             The options for this lookup request.
-        keys (Sequence[google.cloud.datastore_v1.types.Key]):
+        keys (MutableSequence[google.cloud.datastore_v1.types.Key]):
             Required. Keys of entities to look up.
     """
 
-    project_id = proto.Field(
+    project_id: str = proto.Field(
         proto.STRING,
         number=8,
     )
-    database_id = proto.Field(
+    database_id: str = proto.Field(
         proto.STRING,
         number=9,
     )
-    read_options = proto.Field(
+    read_options: "ReadOptions" = proto.Field(
         proto.MESSAGE,
         number=1,
         message="ReadOptions",
     )
-    keys = proto.RepeatedField(
+    keys: MutableSequence[entity.Key] = proto.RepeatedField(
         proto.MESSAGE,
         number=3,
         message=entity.Key,
@@ -92,40 +94,50 @@ class LookupResponse(proto.Message):
     [Datastore.Lookup][google.datastore.v1.Datastore.Lookup].
 
     Attributes:
-        found (Sequence[google.cloud.datastore_v1.types.EntityResult]):
+        found (MutableSequence[google.cloud.datastore_v1.types.EntityResult]):
             Entities found as ``ResultType.FULL`` entities. The order of
             results in this field is undefined and has no relation to
             the order of the keys in the input.
-        missing (Sequence[google.cloud.datastore_v1.types.EntityResult]):
+        missing (MutableSequence[google.cloud.datastore_v1.types.EntityResult]):
             Entities not found as ``ResultType.KEY_ONLY`` entities. The
             order of results in this field is undefined and has no
             relation to the order of the keys in the input.
-        deferred (Sequence[google.cloud.datastore_v1.types.Key]):
+        deferred (MutableSequence[google.cloud.datastore_v1.types.Key]):
             A list of keys that were not looked up due to
             resource constraints. The order of results in
             this field is undefined and has no relation to
             the order of the keys in the input.
+        transaction (bytes):
+            The identifier of the transaction that was started as part
+            of this Lookup request.
+
+            Set only when [ReadOptions.begin_transaction][] was set in
+            [LookupRequest.read_options][google.datastore.v1.LookupRequest.read_options].
         read_time (google.protobuf.timestamp_pb2.Timestamp):
             The time at which these entities were read or
             found missing.
     """
 
-    found = proto.RepeatedField(
+    found: MutableSequence[gd_query.EntityResult] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=gd_query.EntityResult,
     )
-    missing = proto.RepeatedField(
+    missing: MutableSequence[gd_query.EntityResult] = proto.RepeatedField(
         proto.MESSAGE,
         number=2,
         message=gd_query.EntityResult,
     )
-    deferred = proto.RepeatedField(
+    deferred: MutableSequence[entity.Key] = proto.RepeatedField(
         proto.MESSAGE,
         number=3,
         message=entity.Key,
     )
-    read_time = proto.Field(
+    transaction: bytes = proto.Field(
+        proto.BYTES,
+        number=5,
+    )
+    read_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=7,
         message=timestamp_pb2.Timestamp,
@@ -171,31 +183,31 @@ class RunQueryRequest(proto.Message):
             This field is a member of `oneof`_ ``query_type``.
     """
 
-    project_id = proto.Field(
+    project_id: str = proto.Field(
         proto.STRING,
         number=8,
     )
-    database_id = proto.Field(
+    database_id: str = proto.Field(
         proto.STRING,
         number=9,
     )
-    partition_id = proto.Field(
+    partition_id: entity.PartitionId = proto.Field(
         proto.MESSAGE,
         number=2,
         message=entity.PartitionId,
     )
-    read_options = proto.Field(
+    read_options: "ReadOptions" = proto.Field(
         proto.MESSAGE,
         number=1,
         message="ReadOptions",
     )
-    query = proto.Field(
+    query: gd_query.Query = proto.Field(
         proto.MESSAGE,
         number=3,
         oneof="query_type",
         message=gd_query.Query,
     )
-    gql_query = proto.Field(
+    gql_query: gd_query.GqlQuery = proto.Field(
         proto.MESSAGE,
         number=7,
         oneof="query_type",
@@ -213,18 +225,28 @@ class RunQueryResponse(proto.Message):
         query (google.cloud.datastore_v1.types.Query):
             The parsed form of the ``GqlQuery`` from the request, if it
             was set.
+        transaction (bytes):
+            The identifier of the transaction that was started as part
+            of this RunQuery request.
+
+            Set only when [ReadOptions.begin_transaction][] was set in
+            [RunQueryRequest.read_options][google.datastore.v1.RunQueryRequest.read_options].
     """
 
-    batch = proto.Field(
+    batch: gd_query.QueryResultBatch = proto.Field(
         proto.MESSAGE,
         number=1,
         message=gd_query.QueryResultBatch,
     )
-    query = proto.Field(
+    query: gd_query.Query = proto.Field(
         proto.MESSAGE,
         number=2,
         message=gd_query.Query,
     )
+    transaction: bytes = proto.Field(
+        proto.BYTES,
+        number=5,
+    )
 
 
 class RunAggregationQueryRequest(proto.Message):
@@ -266,31 +288,31 @@ class RunAggregationQueryRequest(proto.Message):
             This field is a member of `oneof`_ ``query_type``.
     """
 
-    project_id = proto.Field(
+    project_id: str = proto.Field(
         proto.STRING,
         number=8,
     )
-    database_id = proto.Field(
+    database_id: str = proto.Field(
         proto.STRING,
         number=9,
     )
-    partition_id = proto.Field(
+    partition_id: entity.PartitionId = proto.Field(
         proto.MESSAGE,
         number=2,
         message=entity.PartitionId,
     )
-    read_options = proto.Field(
+    read_options: "ReadOptions" = proto.Field(
         proto.MESSAGE,
         number=1,
         message="ReadOptions",
     )
-    aggregation_query = proto.Field(
+    aggregation_query: gd_query.AggregationQuery = proto.Field(
         proto.MESSAGE,
         number=3,
         oneof="query_type",
         message=gd_query.AggregationQuery,
     )
-    gql_query = proto.Field(
+    gql_query: gd_query.GqlQuery = proto.Field(
         proto.MESSAGE,
         number=7,
         oneof="query_type",
@@ -309,18 +331,28 @@ class RunAggregationQueryResponse(proto.Message):
         query (google.cloud.datastore_v1.types.AggregationQuery):
             The parsed form of the ``GqlQuery`` from the request, if it
             was set.
+        transaction (bytes):
+            The identifier of the transaction that was started as part
+            of this RunAggregationQuery request.
+
+            Set only when [ReadOptions.begin_transaction][] was set in
+            [RunAggregationQueryRequest.read_options][google.datastore.v1.RunAggregationQueryRequest.read_options].
     """
 
-    batch = proto.Field(
+    batch: aggregation_result.AggregationResultBatch = proto.Field(
         proto.MESSAGE,
         number=1,
         message=aggregation_result.AggregationResultBatch,
     )
-    query = proto.Field(
+    query: gd_query.AggregationQuery = proto.Field(
         proto.MESSAGE,
         number=2,
         message=gd_query.AggregationQuery,
     )
+    transaction: bytes = proto.Field(
+        proto.BYTES,
+        number=5,
+    )
 
 
 class BeginTransactionRequest(proto.Message):
@@ -340,15 +372,15 @@ class BeginTransactionRequest(proto.Message):
             Options for a new transaction.
     """
 
-    project_id = proto.Field(
+    project_id: str = proto.Field(
         proto.STRING,
         number=8,
     )
-    database_id = proto.Field(
+    database_id: str = proto.Field(
         proto.STRING,
         number=9,
     )
-    transaction_options = proto.Field(
+    transaction_options: "TransactionOptions" = proto.Field(
         proto.MESSAGE,
         number=10,
         message="TransactionOptions",
@@ -364,7 +396,7 @@ class BeginTransactionResponse(proto.Message):
             The transaction identifier (always present).
     """
 
-    transaction = proto.Field(
+    transaction: bytes = proto.Field(
         proto.BYTES,
         number=1,
     )
@@ -388,15 +420,15 @@ class RollbackRequest(proto.Message):
             [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
     """
 
-    project_id = proto.Field(
+    project_id: str = proto.Field(
         proto.STRING,
         number=8,
     )
-    database_id = proto.Field(
+    database_id: str = proto.Field(
         proto.STRING,
         number=9,
     )
-    transaction = proto.Field(
+    transaction: bytes = proto.Field(
         proto.BYTES,
         number=1,
     )
@@ -414,6 +446,10 @@ class CommitRequest(proto.Message):
     r"""The request for
     [Datastore.Commit][google.datastore.v1.Datastore.Commit].
 
+    This message has `oneof`_ fields (mutually exclusive fields).
+    For each oneof, at most one member field can be set at the same time.
+    Setting any member of the oneof automatically clears all other
+    members.
 
     .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields
 
@@ -435,7 +471,16 @@ class CommitRequest(proto.Message):
             [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
 
             This field is a member of `oneof`_ ``transaction_selector``.
-        mutations (Sequence[google.cloud.datastore_v1.types.Mutation]):
+        single_use_transaction (google.cloud.datastore_v1.types.TransactionOptions):
+            Options for beginning a new transaction for this request.
+            The transaction is committed when the request completes. If
+            specified,
+            [TransactionOptions.mode][google.datastore.v1.TransactionOptions.mode]
+            must be
+            [TransactionOptions.ReadWrite][google.datastore.v1.TransactionOptions.ReadWrite].
+
+            This field is a member of `oneof`_ ``transaction_selector``.
+        mutations (MutableSequence[google.cloud.datastore_v1.types.Mutation]):
             The mutations to perform.
 
             When mode is ``TRANSACTIONAL``, mutations affecting a single
@@ -458,25 +503,31 @@ class CommitRequest(proto.Message):
         TRANSACTIONAL = 1
         NON_TRANSACTIONAL = 2
 
-    project_id = proto.Field(
+    project_id: str = proto.Field(
         proto.STRING,
         number=8,
     )
-    database_id = proto.Field(
+    database_id: str = proto.Field(
         proto.STRING,
         number=9,
     )
-    mode = proto.Field(
+    mode: Mode = proto.Field(
         proto.ENUM,
         number=5,
         enum=Mode,
     )
-    transaction = proto.Field(
+    transaction: bytes = proto.Field(
         proto.BYTES,
         number=1,
         oneof="transaction_selector",
     )
-    mutations = proto.RepeatedField(
+    single_use_transaction: "TransactionOptions" = proto.Field(
+        proto.MESSAGE,
+        number=10,
+        oneof="transaction_selector",
+        message="TransactionOptions",
+    )
+    mutations: MutableSequence["Mutation"] = proto.RepeatedField(
         proto.MESSAGE,
         number=6,
         message="Mutation",
@@ -488,7 +539,7 @@ class CommitResponse(proto.Message):
     [Datastore.Commit][google.datastore.v1.Datastore.Commit].
 
     Attributes:
-        mutation_results (Sequence[google.cloud.datastore_v1.types.MutationResult]):
+        mutation_results (MutableSequence[google.cloud.datastore_v1.types.MutationResult]):
             The result of performing the mutations.
             The i-th mutation result corresponds to the i-th
             mutation in the request.
@@ -500,16 +551,16 @@ class CommitResponse(proto.Message):
             non-transactional commits.
     """
 
-    mutation_results = proto.RepeatedField(
+    mutation_results: MutableSequence["MutationResult"] = proto.RepeatedField(
         proto.MESSAGE,
         number=3,
         message="MutationResult",
     )
-    index_updates = proto.Field(
+    index_updates: int = proto.Field(
         proto.INT32,
         number=4,
     )
-    commit_time = proto.Field(
+    commit_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=8,
         message=timestamp_pb2.Timestamp,
@@ -529,21 +580,21 @@ class AllocateIdsRequest(proto.Message):
             the request.
             '(default)' is not allowed; please use empty
             string '' to refer the default database.
-        keys (Sequence[google.cloud.datastore_v1.types.Key]):
+        keys (MutableSequence[google.cloud.datastore_v1.types.Key]):
             Required. A list of keys with incomplete key
             paths for which to allocate IDs. No key may be
             reserved/read-only.
     """
 
-    project_id = proto.Field(
+    project_id: str = proto.Field(
         proto.STRING,
         number=8,
     )
-    database_id = proto.Field(
+    database_id: str = proto.Field(
         proto.STRING,
         number=9,
     )
-    keys = proto.RepeatedField(
+    keys: MutableSequence[entity.Key] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=entity.Key,
@@ -555,13 +606,13 @@ class AllocateIdsResponse(proto.Message):
     [Datastore.AllocateIds][google.datastore.v1.Datastore.AllocateIds].
 
     Attributes:
-        keys (Sequence[google.cloud.datastore_v1.types.Key]):
+        keys (MutableSequence[google.cloud.datastore_v1.types.Key]):
             The keys specified in the request (in the
             same order), each with its key path completed
             with a newly allocated ID.
     """
 
-    keys = proto.RepeatedField(
+    keys: MutableSequence[entity.Key] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=entity.Key,
@@ -581,21 +632,21 @@ class ReserveIdsRequest(proto.Message):
             the request.
             '(default)' is not allowed; please use empty
             string '' to refer the default database.
-        keys (Sequence[google.cloud.datastore_v1.types.Key]):
+        keys (MutableSequence[google.cloud.datastore_v1.types.Key]):
             Required. A list of keys with complete key
             paths whose numeric IDs should not be
             auto-allocated.
     """
 
-    project_id = proto.Field(
+    project_id: str = proto.Field(
         proto.STRING,
         number=8,
     )
-    database_id = proto.Field(
+    database_id: str = proto.Field(
         proto.STRING,
         number=9,
     )
-    keys = proto.RepeatedField(
+    keys: MutableSequence[entity.Key] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=entity.Key,
@@ -660,36 +711,36 @@ class Mutation(proto.Message):
             This field is a member of `oneof`_ ``conflict_detection_strategy``.
     """
 
-    insert = proto.Field(
+    insert: entity.Entity = proto.Field(
         proto.MESSAGE,
         number=4,
         oneof="operation",
         message=entity.Entity,
     )
-    update = proto.Field(
+    update: entity.Entity = proto.Field(
         proto.MESSAGE,
         number=5,
         oneof="operation",
         message=entity.Entity,
     )
-    upsert = proto.Field(
+    upsert: entity.Entity = proto.Field(
         proto.MESSAGE,
         number=6,
         oneof="operation",
         message=entity.Entity,
     )
-    delete = proto.Field(
+    delete: entity.Key = proto.Field(
         proto.MESSAGE,
         number=7,
         oneof="operation",
         message=entity.Key,
     )
-    base_version = proto.Field(
+    base_version: int = proto.Field(
         proto.INT64,
         number=8,
         oneof="conflict_detection_strategy",
     )
-    update_time = proto.Field(
+    update_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=11,
         oneof="conflict_detection_strategy",
@@ -726,21 +777,21 @@ class MutationResult(proto.Message):
             strategy field is not set in the mutation.
     """
 
-    key = proto.Field(
+    key: entity.Key = proto.Field(
         proto.MESSAGE,
         number=3,
         message=entity.Key,
     )
-    version = proto.Field(
+    version: int = proto.Field(
         proto.INT64,
         number=4,
     )
-    update_time = proto.Field(
+    update_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=6,
         message=timestamp_pb2.Timestamp,
     )
-    conflict_detected = proto.Field(
+    conflict_detected: bool = proto.Field(
         proto.BOOL,
         number=5,
     )
@@ -767,6 +818,16 @@ class ReadOptions(proto.Message):
             transaction identifier is returned by a call to
             [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
 
+            This field is a member of `oneof`_ ``consistency_type``.
+        new_transaction (google.cloud.datastore_v1.types.TransactionOptions):
+            Options for beginning a new transaction for this request.
+
+            The new transaction identifier will be returned in the
+            corresponding response as either
+            [LookupResponse.transaction][google.datastore.v1.LookupResponse.transaction]
+            or
+            [RunQueryResponse.transaction][google.datastore.v1.RunQueryResponse.transaction].
+
             This field is a member of `oneof`_ ``consistency_type``.
         read_time (google.protobuf.timestamp_pb2.Timestamp):
             Reads entities as they were at the given
@@ -783,18 +844,24 @@ class ReadOptions(proto.Message):
         STRONG = 1
         EVENTUAL = 2
 
-    read_consistency = proto.Field(
+    read_consistency: ReadConsistency = proto.Field(
         proto.ENUM,
         number=1,
         oneof="consistency_type",
         enum=ReadConsistency,
     )
-    transaction = proto.Field(
+    transaction: bytes = proto.Field(
         proto.BYTES,
         number=2,
         oneof="consistency_type",
     )
-    read_time = proto.Field(
+    new_transaction: "TransactionOptions" = proto.Field(
+        proto.MESSAGE,
+        number=3,
+        oneof="consistency_type",
+        message="TransactionOptions",
+    )
+    read_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=4,
         oneof="consistency_type",
@@ -839,7 +906,7 @@ class TransactionOptions(proto.Message):
                 being retried.
         """
 
-        previous_transaction = proto.Field(
+        previous_transaction: bytes = proto.Field(
             proto.BYTES,
             number=1,
         )
@@ -853,19 +920,19 @@ class TransactionOptions(proto.Message):
                 This may not be older than 60 seconds.
         """
 
-        read_time = proto.Field(
+        read_time: timestamp_pb2.Timestamp = proto.Field(
             proto.MESSAGE,
             number=1,
             message=timestamp_pb2.Timestamp,
         )
 
-    read_write = proto.Field(
+    read_write: ReadWrite = proto.Field(
         proto.MESSAGE,
         number=1,
         oneof="mode",
         message=ReadWrite,
     )
-    read_only = proto.Field(
+    read_only: ReadOnly = proto.Field(
         proto.MESSAGE,
         number=2,
         oneof="mode",
